
%  SYNOPSIS:
%
%  This MATLAB script computes the distence to uncontrollability for a 
%  linear dynamic system using the OEIG routine.
%
%  The dynamical system is assumed to be of the following form:
%
%  Plant:        dx/dt = A1 x(t) + A2 u(t) 
%  State:        x
%  Control:      u
%
%  Distance to uncontrollability of the above dynamical system is computed 
%  as follows: 
%
%  argmin  sigma_min{[A1 A2]' - mu*[I 0]'}
%    mu
%
%  Here, sigma_min is the smallest singular value of its matrix argument.
%
%  In the framework of overdetermined eigenvalue problem, the above solution
%  is one of the solution of OEIG([A1 A2]', [I 0]').
%
%  Note that OEIG computes all the local minima. However, in the contex
%  of distance to uncontrollability the global minimum is required. The 
%  eigenvalue 'mu' for which the smallest singular value of 
%  ([A1'; A2'] - mu*[I, 0]) is minimal is thus the desired solution.
%  

%  AUTHOR / BUG REPORT:
%
%    saptarshi.das@univie.ac.at

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  MATLAB SPECIFIC SETUP
%

clear all
format long e
setup

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  EX1 -- Example taken from Section 5 of the following article:
%
%  @article{gu2000new,
%   title={New methods for estimating the distance to uncontrollability},
%   author={Gu, M.},
%   journal={SIAM Journal on Matrix Analysis and Applications},
%   volume={21},
%   number={3},
%   pages={989--1003},
%   year={2000},
% }
%

%%  Set the matrices of the dynamical system.

A1 = [ 1  1  2  3; ...
      -1  1  4  5; ...
       0  0  1  2; ...
       0  0 -2  1];

A2 = [ 1  1  0  0]';

%%  Setup the matrix pencil (A, B)

A  = [A1 A2]';
B  = [eye(4, 4); zeros(1, 4)];

%%  Solve overdetermined eigenvalue problem
mu = oeig(A, B);

%%  Find the smallest singular value. 
%%
%%  NOTE: for practical purpose, this distance should
%%  be computed as the value of the objective function of
%%  the overdetermined eigenvalue problem.

sig_n = -1*ones(4, 1);
for n = 1 : 4
  sig = svd(A - mu(n)*B);
  sig_n(n) = sig(4); 
end

ducon = min(sig_n);

clc; fprintf('\n');
fprintf('EXAMPLE 1: taken from Gu (2000), SIMAX, vol 21, no. 3. \n\n')
fprintf('Plant:        dx/dt = A1 x(t) + A2 u(t) \n'); 
fprintf('State:        x \n');
fprintf('Control:      u \n');
A1
A2
fprintf('Distance to uncontrollability: %f \n', ducon);
fprintf('This is an example of uncontrollable system. \n\n');
fprintf('For next example press any key \n\n');
pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  EX2 -- Example taken from Section 5 of the following article:
%
% @article{he1995estimating,
%   title={Estimating the distance to uncontrollability: 
%          A fast method and a slow one},
%   author={He, C.},
%   journal={Systems \& control letters},
%   volume={26},
%   number={4},
%   pages={275--281},
%   year={1995},
%   publisher={Elsevier}
% }
%

%%  Set the matrices of the dynamical system.

A1 = [ 1    1  1; ...
      0.1   3  5; ...
       0   -1 -1];

A2 = [1 0.1 0]';

%%  Setup the matrix pencil (A, B)

A  = [A1'; A2'];
B  = [eye(3, 3); zeros(1, 3)];

%%  Solve overdetermined eigenvalue problem
mu = oeig(A, B);

%%  Find the smallest singular value. 
%%
%%  NOTE: for practical purpose, this distance should
%%  be computed as the value of the objective function of
%%  the overdetermined eigenvalue problem.

sig_n = -1*ones(3, 1);
for n = 1 : 3
  sig = svd(A - mu(n)*B);
  sig_n(n) = sig(3); 
end

ducon = min(sig_n);

fprintf('EXAMPLE 2: taken from He (1995), Systems and Control letters,\n') 
fprintf('vol 26, no. 4. \n\n')
fprintf('Plant:        dx/dt = A1 x(t) + A2 u(t) \n'); 
fprintf('State:        x \n');
fprintf('Control:      u \n');
A1
A2
fprintf('Distance to uncontrollability: %f \n', ducon);
fprintf('Result using OEIG matches exactly with published results. \n\n');


%  CHANGE LOG:
%
%  ** Aug. 15, 2011 -- formatted output of two examples.
%  ** Aug. 14, 2011 -- first key in.
