%
%  SYNOPSIS:
%
%    A MATLAB script to illustrate the growing accuracy with increasing 
%    number of rows in the context of overdetermined eigenvalue problems. 
%
%    The following tasks are sequentially performed:
%
%    0. Setup paths, clear variables etc...
%    1. Select methods and variables.
%    2. Simulate overdetermined matrices with given eigenpairs, and perturb.
%    3. Get starting eigenpairs.
%    4. Filter eigenpairs starting routines.
%    5. For different matrices of growing size (by row selection)
%      5. Estimate eigenpairs of overdetermined matrices. 
%      6. Note the error with the noisefree eigenvalues. 
%    7. Demonstrate results (plots).
%

%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%
%  LAST MODIFICATION:
%
%  Feb. 18, 2011, iterating over different rows
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SETUP
%
%  ** the setup routine sets the path, if you want to check
%     any dependency (like existance of toolbox) do it from 
%     the setup script.
%
%  ** the seeds of the random number generator are fixed so as to
%     can be used with different methods and parameters related to the
%     eigenpair estimation algorithm. If you want to get results through
%     statistical simulation, then delete/comment the related lines. 

clear all
format short e
setup
rand('seed', 0);
randn('seed', 0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT PARAMETERS FOR SIMULATION 
%
%  ** Select parameters for simulation of overdetermined matrix pairs 
%     with prescribed generalized eigenvalues, 'eigs'. Furthur the entries
%     of the matrix pair is perturbed with additive white Gaussian noise
%     of sigma equal to 'sig'.
%
%  N_rows      number of rows over which the simulation will be performed.
%
%  [M, N]      size of the overdetermined matrix pairs. NOTE: M >= N. 
%              At first a big matrix of the maximum size is generated,
%              later sub-matrices are formed (by selecting rows as specified
%              in the N_rows vector). 
%
%  eigs        generalized eigenvalues for the overdetermined matrix pairs, 
%              this will be used by the simulation routine. Useful for
%              testing backward stabiltiy. NOTE: length(eigs) <= N. 
%
%  sig         sigma of additive white Gaussian noise used to perturb the
%              overdetermined matrix pairs. This is done so that the
%              simulated overdetermined matrix pairs get no exact solution.
%
%  N_sim       Number of runs using different random noise sequence with
%              standard deviation equal to sig.

N_rows = [5:2:31];
M = max(N_rows); N = 5;

eigs = [2 + sqrt(-1)*4; ...
        3 + sqrt(-1)*2;  ...
        4 + sqrt(-1)*2.2];

sig  = .03;

N_sim = 1000;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT METHODS FOR STARTING EIGENPAIRS 
%
%  ** Select one method for generating the starting eigenpairs. Available
%     methods are listed below. Note that if you select 'ini_method 4', then
%     you also have to provide initial eigenvalues.
%
%  'ini_method 0': if user have a good guess about the initial eigenpairs,
%                  the use this option. Do not forget to set the initial
%                  eigenvectors in the columns of matrix X_0, and eigenvalues
%                  in the diagonal matrix D_0.
%
%  'ini_method 1': from random choice of rows to form a square subset of 
%                  rectangular matrices. (not recommended) 
%
%  'ini_method 2': from QR decomposition of [B, A]. (highly recommended)
%
%  'ini_method 3': from the square matrix pencil [B'A, B'B]. (Suggested by
%                  Boutry et al. 2005, SIMAX Vol. 27, No. 2)
%
%  'ini_method 4': if rough estimates of eigenvalues (lambda) are known, then  
%                  rough estimates of eigenvectors are smallest right singular 
%                  vector of (A - lambda*B).
%

ini_method = 'ini_method 2';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT PARAMETER TO FILTER EIGENPAIRS 
%
%  ** if you do not want to filter eigenpairs before starting the iterative
%     methods, then set the variable to zero (0).  
%
%  ** you can define your own criterion for selecting eigenpairs. 
%
%  thr   drop an eigenpair if the abs(1/eigenvalue) is smaller than thr.

thr = .1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT PARAMETERS FOR THE EIGENPAIR ESTIMATION ALGORITHM
%
%  ** this script will execute the selected method. The following methods
%     are available in this software for estimation of generalized eigenpairs 
%     of overdetermined matrix pairs.
%
%     'MPA'     minimum perturbation approach by Boutry et al., SIAMX 2006.    
%               Use ">>help get_eig_MPA" to know more.
%
%     'direct'  direct optimization of the objective function. 
%               Use ">>help get_eig_direct" to know more.
%
%     'ldist'   method based on least distance approach.
%               Use ">>help get_eig_least_distance" to know more.
%
%     'joint'   joint optimization of eigenvalues and eigenvectors.
%               Use ">>help get_eig_joint" to know more.
%
%  ** generally the first three mathods takes a lot of iterations to 
%     converge. Some time around order of 10,000. The last method based
%     on joint optimization of eigenvalue and eigenvector works much much 
%     faster.
%
% 
%  ** max_iter       set the maximum number of iterations. 
%

esti_method = 'joint';
min_iter    = 50; 
max_iter    = 50; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  PARAMETER AND METHOD SETTING DONE.
%  EXECUTING THE ROUTINES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SIMULATION, AND PERTURBATION
%
%  ** simulate overdetermined matrix pair (A, B) with given
%     eigenvalues and dimensions. The matrix 'X' contains
%     the generalized eigenvectors in its columns.
%
%  ** perturb entries of the matrix pair (A, B) with additive white
%     Gaussian noise with sigma equal to 'sig'.
%
%  ** if one have the data already measured, then set it to 
%     A and B, and skip the simulation part.

[A_ef, B_ef, X] = get_matrices_from_gen_eigvals(M, N, eigs);

err = zeros(N_sim, length(N_rows));
for n_sim = 1 : N_sim 
  n_sim
  A = A_ef + .5*sig^2*(randn(M, N) + sqrt(-1)*randn(M, N));
  B = B_ef + .5*sig^2*(randn(M, N) + sqrt(-1)*randn(M, N));
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  GET STARTING EIGENPAIRS 
  %
  %  * depending on the method selected for estimating 
  %    the starting eigenpairs, the relevent routine is
  %    called.
  %
  %  * if one have a good guess of starting eigenpairs, 
  %    then set the eigenvectors in the columns of X_0,
  %    and eigenvalues in the diagonal matrix D_0. And
  %    skip the in-bulit routines. Do not forget to set
  %    ini_method to 'ini_method 0'.
  %
  %  * if one have a good guess of starting eigenvalues 
  %    only, and want only the corresponding eigenvectors
  %    to be computed, the set ini_method to 'ini_method 4',
  %    and set the eigenvalues in the column matrix E_0.
  
  switch ini_method
    case {'ini_method 0'}
      if (~exist('X_0', 'var') || ~exist('D_0', 'var'))
        error('missing:data', 'initial eigen pairs are missing')
      end
    case {'ini_method 1'}
      [X_0, D_0] = initial_eigenpairs_using_random_rows(A, B);
    case {'ini_method 2'}
      [X_0, D_0] = initial_eigenpairs_using_QR_decomp(A, B);
    case {'ini_method 3'}
      [X_0, D_0] = initial_eigenpairs_using_square_form(A, B);
    case {'ini_method 4'}
      if (~exist('E_0', 'var'))
        error('missing:data', 'initial eigen pairs are missing')
      end
      X_0 = initial_eigenvectors_from_smallest_singularvalue(A, B, E_0);
      D_0 = diag(E_0);
    otherwise
      error('invalid:option', 'the option for initial eigenpairs is not valid')
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  FILTER EIGENPAIRS BEFORE ITERATIVE ALGORITHMS
  %
  %  thr   drop an eigenpair if the abs(1/eigenvalue) is smaller than thr.
  %  N_e   number of initial eigenpairs to process.
  
  [X_0, D_0] = filter_initial_eigenpairs(X_0, D_0, thr);
  N_e  = length(diag(D_0));
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %
  %  ESTIMATE EIGENPAIRS : MINIMAL PERTURNATION APPROACH, MPA
  %  ESTIMATE EIGENPAIRS : DIRECT APPROACH
  %  ESTIMATE EIGENPAIRS : LEAST DISTANCE FOR LOW COMPLEXITY
  %  ESTIMATE EIGENPAIRS : JOINT OPTIMIZATION OF EIGENVALUES AND VECTORS
  %
  
  
  for i =  1 : length(N_rows) 
    n_rows = N_rows(i); 
    %%  truncate the matrices to get a sub matrix 
    %%  with n_rows rows 
    A_t = A(1:n_rows, :);
    B_t = B(1:n_rows, :);
  
    switch esti_method
      case {'MPA'}
        [V, D, D_all] = get_eig_MPA(A_t, B_t, diag(D_0), min_iter, max_iter);
      case {'direct'}
        [V, D, D_all] = get_eig_direct(A_t, B_t, X_0, D_0, min_iter, max_iter);
      case {'ldist'}
        [V, D, D_all] = get_eig_least_distance(A_t, B_t, X_0, D_0, ...
                                    min_iter, max_iter);
      case {'joint'}
        [V, D, D_all] = get_eig_joint(A_t, B_t, X_0, D_0, min_iter, max_iter);
      otherwise
        error('invalid:option', ...
              'the option for eigenpair estimation method is not valid')
    end
    %% Find the error with one of the noiseless eigenvalue.
    %% Without loss of generaliety, we choose the first one.
    err(n_sim, i) = min(abs(diag(D) - eigs(1)))^2;
  end
end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  PLOTS AND RESULTS
%

fullscreen = get(0,'ScreenSize');
figure('Position',[0 150 fullscreen(3) fullscreen(4)])
semilogy(N_rows, (mean(err)).^(.5), '*-')
xlabel('Number of rows')
ylabel('RMSE')
grid on
% plot_finish
saveas(gcf, 'acc_vs_nrows', 'eps')

return
%% Plot level curves 
figure()
plot(eigs, 'bs', 'markersize', 6)
hold on
plot(diag(D_0), 'ro', 'markersize', 6)
plot(diag(D(1:N_e, 1:N_e)), 'r*', 'markersize', 6)

legend('exact', 'starting', 'estimated');

title('The position of the estimated eigenvalues in complex plane', ...
       'fontweight', 'bold', 'fontsize', 12)
axis square 
axis equal
% plot_finish

%% Plot the path traced
plot(D_all.', '-')

%% Plot the level curves of the function optimized 
xdelta = .01;
ydelta = .01;
xmin = min(real(diag(D))) - .25;
xmax = max(real(diag(D))) + .25;
ymin = min(imag(diag(D))) - .25;
ymax = max(imag(diag(D))) + .25;
srf = plot_rect_eig_surf(A, B, xmin, xmax, ymin, ymax, xdelta, ydelta);
axis([xmin xmax ymin ymax])

plot_finish


