%
%  SYNOPSIS:
%
%    This MATLAB script demonstrates the usage of routies (functions) of the
%    software overdeteig designed for estimating eigenpairs 
%    (eigenvalues, eigenvectors) of a overdetermined pair of matrices. 
%    Apart from estimating eigenpairs, the software provides routines for
%    simulating overdetermined matrix pairs with given eigenpairs, 
%    routines for compputing starting eigenpairs, and routines for
%    visualization.
%
%    The following tasks are sequentially demonstrated and executed:
%
%    0. Setup paths, clear variables etc...
%    1. Select methods and variables.
%    2. Simulate overdetermined matrices with given eigenpairs, and perturb.
%    3. Get starting eigenpairs.
%    4. Filter eigenpairs starting routines.
%    5. Estimate eigenpairs of overdetermined matrices. 
%    6. Demonstrate results (plots).
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%
%  LAST MODIFICATION:
%
%  Jan. 12, 2011, more user friendly design to execute routines. 
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SETUP
%
%  ** the setup routine sets the path, if you want to check
%     any dependency (like existance of toolbox) do it from 
%     the setup script.
%
%  ** the seeds of the random number generator are fixed so as to
%     simulate the same matrix pairs on each run, and the same matrix
%     can be used with different methods and parameters related to the
%     eigenpair estimation algorithm. If you want to get results through
%     statistical simulation, then delete/comment the related lines. 

clear all
format short e
setup
rand('seed', 0);
randn('seed', 0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT PARAMETERS FOR SIMULATION 
%
%  ** Select parameters for simulation of overdetermined matrix pairs 
%     with prescribed generalized eigenvalues, 'eigs'. Furthur the entries
%     of the matrix pair is perturbed with additive white Gaussian noise
%     of sigma equal to 'sig'.
%
%  [M, N]      size of the overdetermined matrix pairs. NOTE: M >= N. 
%
%  eigs        generalized eigenvalues for the overdetermined matrix pairs, 
%              this will be used by the simulation routine. Useful for
%              testing backward stabiltiy. NOTE: lenght(eigs) <= N. 
%
%  sig         sigma of additive white Gaussian noise used to perturb the
%              overdetermined matrix pairs. This is done so that the
%              simulated overdetermined matrix pairs get no exact solution.

M = 50; N = 5;

eigs = [2 + sqrt(-1)*4; ...
        3 + sqrt(-1)*2;  ...
        4 + sqrt(-1)*2.2];

%eigs = [-1 + sqrt(-1)*1; ...
%        -.5 + sqrt(-1)*2;  ...
%        -1 + sqrt(-1)*.5];

sig  = .01;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT METHODS FOR STARTING EIGENPAIRS 
%
%  ** Select one method for generating the starting eigenpairs. Available
%     methods are listed below. Note that if you select 'ini_method 4', then
%     you also have to provide initial eigenvalues.
%
%  'ini_method 0': if user have a good guess about the initial eigenpairs,
%                  the use this option. Do not forget to set the initial
%                  eigenvectors in the columns of matrix X_0, and eigenvalues
%                  in the diagonal matrix D_0.
%
%  'ini_method 1': from random choice of rows to form a square subset of 
%                  rectangular matrices. (not recommended) 
%
%  'ini_method 2': from QR decomposition of [B, A]. (highly recommended)
%
%  'ini_method 3': from the square matrix pencil [B'A, B'B]. (Suggested by
%                  Boutry et al. 2005, SIMAX Vol. 27, No. 2)
%
%  'ini_method 4': if rough estimates of eigenvalues (lambda) are known, then  
%                  rough estimates of eigenvectors are smallest right singular 
%                  vector of (A - lambda*B).
%

ini_method = 'ini_method 2';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT PARAMETER TO FILTER EIGENPAIRS 
%
%  ** if you do not want to filter eigenpairs before starting the iterative
%     methods, then set the variable to zero (0).  
%
%  ** you can define your own criterion for selecting eigenpairs. 
%
%  thr   drop an eigenpair if the abs(1/eigenvalue) is smaller than thr.

thr = .1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SELECT PARAMETERS FOR THE EIGENPAIR ESTIMATION ALGORITHM
%
%  ** this script will execute all the methods available in this software
%     for estimation of generalized eigenpairs of overdetermined matrix
%     pairs.
%
%  ** generally the first three mathods takes a lot of iterations to 
%     converge. Some time around order of 10,000. The last method based
%     on joint optimization of eigenvalue and eigenvector works much much 
%     faster.
%
%  max_iter_MPA     maximum number of iterations for algorithm based on 
%                   minimal perturbation approach. 
%
%  max_iter_direct  maximum number of iterations for algorithm based on 
%                   direct optimization method.
%
%  max_iter_ldist   maximum number of iterations for algorithm based on 
%                   the least distance approach.
%
%  max_iter_joint   maximum number of iterations for algorithm based on join
%                   optimzation of eigenvalue and eigenvector. 
%                   (recommened method)
%
%  min_iter_[MPA, direct, ldist, joint] minimum number of iterations for each
%                   of the methods.

min_iter_MPA    = 10^1; 
min_iter_direct = 10^1;
min_iter_ldist  = 10^4;
min_iter_joint  = 30;

max_iter_MPA    = 10^1; 
max_iter_direct = 10^1;
max_iter_ldist  = 10^4;
max_iter_joint  = 10^2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  PARAMETER AND METHOD SETTING DONE.
%  EXECUTING THE ROUTINES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SIMULATION, AND PERTURBATION
%
%  ** simulate overdetermined matrix pair (A, B) with given
%     eigenvalues and dimensions. The matrix 'X' contains
%     the generalized eigenvectors in its columns.
%
%  ** perturb entries of the matrix pair (A, B) with additive white
%     Gaussian noise with sigma equal to 'sig'.
%
%  ** if one have the data already measured, then set it to 
%     A and B, and skip the simulation part.

[A, B, X] = get_matrices_from_gen_eigvals(M, N, eigs);

A = A + .5*sig^2*(randn(M, N) + sqrt(-1)*randn(M, N));
B = B + .5*sig^2*(randn(M, N) + sqrt(-1)*randn(M, N));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  GET STARTING EIGENPAIRS 
%
%  * depending on the method selected for estimating 
%    the starting eigenpairs, the relevent routine is
%    called.
%
%  * if one have a good guess of starting eigenpairs, 
%    then set the eigenvectors in the columns of X_0,
%    and eigenvalues in the diagonal matrix D_0. And
%    skip the in-bulit routines. Do not forget to set
%    ini_method to 'ini_method 0'.
%
%  * if one have a good guess of starting eigenvalues 
%    only, and want only the corresponding eigenvectors
%    to be computed, the set ini_method to 'ini_method 4',
%    and set the eigenvalues in the column matrix E_0.

switch ini_method
  case {'ini_method 0'}
    if (~exist('X_0', 'var') || ~exist('D_0', 'var'))
      error('missing:data', 'initial eigen pairs are missing')
    end
  case {'ini_method 1'}
    [X_0, D_0] = initial_eigenpairs_using_random_rows(A, B);
  case {'ini_method 2'}
    [X_0, D_0] = initial_eigenpairs_using_QR_decomp(A, B);
  case {'ini_method 3'}
    [X_0, D_0] = initial_eigenpairs_using_square_form(A, B);
  case {'ini_method 4'}
    if (~exist('E_0', 'var'))
      error('missing:data', 'initial eigen pairs are missing')
    end
    X_0 = initial_eigenvectors_from_smallest_singularvalue(A, B, E_0);
    D_0 = diag(E_0);
  otherwise
    error('invalid:option', 'the option for initial eigenpairs is not valid')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  FILTER EIGENPAIRS BEFORE ITERATIVE ALGORITHMS
%
%  thr   drop an eigenpair if the abs(1/eigenvalue) is smaller than thr.
%  N_e   number of initial eigenpairs to process.

[X_0, D_0] = filter_initial_eigenpairs(X_0, D_0, thr);
N_e  = length(diag(D_0));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ESTIMATE EIGENPAIRS : MINIMAL PERTURNATION APPROACH, MPA
%  ESTIMATE EIGENPAIRS : REGULARIZATION APPROACH
%  ESTIMATE EIGENPAIRS : LEAST DISTANCE FOR LOW COMPLEXITY
%  ESTIMATE EIGENPAIRS : JOINT OPTIMIZATION OF EIGENVALUES AND VECTORS
%

[V_MPA, D_MPA, D_MPA_all] = get_eig_MPA(A, B, diag(D_0), ...
                            min_iter_MPA, max_iter_MPA);

[V_dir, D_dir, D_dir_all] = get_eig_direct(A, B, X_0, D_0, ... 
                            min_iter_direct, max_iter_direct);

[V_ldist, D_ldist, D_ldist_all] = get_eig_least_distance(A, B, X_0, D_0, ...
                              min_iter_ldist, max_iter_ldist);

[V_joint, D_joint, D_joint_all] = get_eig_joint(A, B, X_0, D_0, ...
                              min_iter_joint, max_iter_joint);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  PLOTS AND RESULTS
%

fullscreen = get(0,'ScreenSize');
figure('Position',[0 150 fullscreen(3) fullscreen(4)])

%% Plot all the final eigenvalues
plot(eigs, 'bs', 'markersize', 10)
hold on
plot(diag(D_0), 'r.', 'markersize', 10)
plot(diag(D_MPA(1:N_e, 1:N_e)), 'ro', 'markersize', 10)
plot(diag(D_dir(1:N_e, 1:N_e)), 'kx', 'markersize', 10)
plot(diag(D_ldist(1:N_e, 1:N_e)), 'k+', 'markersize', 10)
plot(diag(D_joint(1:N_e, 1:N_e)), 'kd', 'markersize', 10)

legend('exact', 'starting', 'estimated (MPA)', 'estimated (direct)',...
       'estimated (least dist.)', 'estimated (joint opt)');

title('The position of the estimated eigenvalues in complex plane', ...
       'fontweight', 'bold', 'fontsize', 12)
plot_finish

%% Plot the path traced
plot(D_MPA_all.')
plot(D_dir_all.')
plot(D_ldist_all.')
plot(D_joint_all.')

%% Plot the level curves of the function optimized 
xdelta = .01;
ydelta = .01;
xmin = min(real(diag(D_joint))) - 1;
xmax = max(real(diag(D_joint))) + 1;
ymin = min(imag(diag(D_joint))) - 1;
ymax = max(imag(diag(D_joint))) + 1;
srf = plot_rect_eig_surf(A, B, xmin, xmax, ymin, ymax, xdelta, ydelta);

axis equal
axis square 
