%
%  SYNOPSIS:
%
%  This MATLAB script compares the surfaces of two objective functions
%  over the complex plane, given two overdetermined matrices A and B. 
%  The objective functions are 
%
%                       || (A - lam*B)*v || 
%     f(lam) =  argmin ---------------------- = min(svd(A - lam*B)) ,
%                 v           ||v||
%                                            .............................(1)
%     and 
%                       || (A - lam*B)*v ||         min(svd(A - lam*B))
%     g(lam) =  argmin ----------------------- =  -----------------------,
%                 v   ||v|| * sqrt(1 + |lam|^2)      sqrt(1 + |lam|^2)
%                                            .............................(2)
%
%  The surface 'g' (eqn. (2)) is proposed by Boutry et al. 2005 (SIAM J. 
%  Matrix Anal. App., Vol. 27, No. 2) for estimation of the generalized
%  eigenvalues of overdetermined matrix pairs.
%
%  However 'g' is not a suitable surface for overdetermined matrix pairs, 
%  which are purturbed version of matrix pairs with less eigenvalues then 
%  the number of columns. In this MATLAB script we demonstrate the mentioned
%  fact. The function 'f' is found to be more suitable for such matrices.
%
%  In the paper by Boutry et al. 2005 (SIAM J.  %  Matrix Anal. App., 
%  Vol. 27, No. 2) above mentioned type of overdetermined matrix pairs 
%  were not considered for illustration.
%
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%
%  LAST MODIFICATION:
%
%  Jan. 12, 2011, updated documentation
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SETUP
%

clear all
format short e
setup
rand('seed', 0);
randn('seed', 0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SIMULATION SETUP, SIMULATION, AND PERTURBATION
%
%  [M, N]      rectangular matrix size.
%  sig2        perturbation in each entry (variance of rand normal). 
%  eigs        set of eigenvalues, for testing backward stabiltiy.

M    =  50;
N    =  5;
sig  = .01;

eigs = [9];

[A, B, X] = get_matrices_from_gen_eigvals(M, N, eigs);

A = A + .5*sig^2*(randn(M, N) + sqrt(-1)*randn(M, N));
B = B + .5*sig^2*(randn(M, N) + sqrt(-1)*randn(M, N));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  COMPUTE THE SURFACE AND TAKE A SECTION
%

xdelta = .1;
ydelta = .1;
xmin = min(real(eigs)) - 3;
xmax = max(real(eigs)) + 3;
ymin = min(imag(eigs)) - 3;
ymax = max(imag(eigs)) + 3;

figure(); clf
subplot(1, 2, 1)
srf = plot_rect_eig_surf(A, B, xmin, xmax, ymin, ymax, xdelta, ydelta);
axis square
title('f_0 plotted over complex plane', 'fontweight', 'bold', 'fontsize', 12)
plot_finish

subplot(1, 2, 2)
srf_MPA = plot_rect_eig_surf_MPA(A, B, xmin, xmax, ymin, ymax, xdelta, ydelta);
axis square
title('g_0 plotted over complex plane', 'fontweight', 'bold', 'fontsize', 12)

plot_finish
colormap gray 

