function [V, D] = initial_eigenpairs_using_QR_decomp(A, B);

%
%  CALLING SEQUENCE:
%
%    [V, D] = initial_eigenpairs_using_QR_decomp(A, B)
%
%  SYNOPSIS:
%
%    This function computes initial eigenpairs (eigenvalues, and eigenvectors)
%    of an overdermined eigenvalue problem. A QR decomposition of the
%    concatenated matrix [A, B] is done to seperate the noise part. The
%    hypothetically no-noise part corresponding to A and B are square 
%    matrices. The generalized eigenvalues and eigenvectors of the no-noise
%    square matrix pencil is returned.
%    
%  INPUT:
%
%    A             a over determined (rectangular) matrix. 
%    B             a over determined (rectangular) matrix. 
%
%  OUTPUT:
%
%    V              a matrix with the eigenvectors in its columns.
%    D              a diagonal matrix with estimated eigenvalues as entries.
%
%  LAST MODIFICATION:
%
%    Jan. 12, 2011, a line on sanity check.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  GET DIMENSIONS 

[M, N] = size(A);
if ([M, N] ~= size(B)) 
  error('dim:missmatch', 'A and B should be of same size')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  QR DECOMPOSITION OF [B, A] GIVES HYPOTHTICALLY NO NOISE SQUARE
%  COUNTERPART OF B AND A. 
%

[Q_tmp, R] = qr([B, A]);
R_11 = triu(R(1:N, 1:N));
R_12 = R(1:N, N+1:2*N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  RETURN EIGEN PAIR OF SQUARE MATRIX PENCIL
%
[V, D] = eig(R_12, R_11);
