function V = initial_eigenvectors_from_smallest_singularvalue(A, B, E_0);

%
%  CALLING SEQUENCE:
%
%  V = initial_eigenvectors_from_smallest_singularvalue(A, B);
%
%  SYNOPSIS:
%
%    This function computes initial eigenvectors of the rectangular matrix
%    pair (A, B) from an initial guess of eigenvalues. The initial guess 
%    of the eigenvalues are passed through the column vector E_0. The 
%    eigenvector of the matrix pencil corresponding to an eigenvalue lambda is
%    the smallest right singular vector of the matrix A - lambda*B. 
%    
%  INPUT:
%
%    A             a over determined (rectangular) matrix. 
%    B             a over determined (rectangular) matrix. 
%    E_0           initial guess of the eigenvalues.
%
%  OUTPUT:
%
%    V              a matrix with the eigenvectors in its columns.
%
%  LAST MODIFICATION:
%
%    Dec. 25, 2010, first lines of codes.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  GET DIMENSIONS AND INITIALIZE
%

N_eig  = length(E_0);
[M, N] = size(A);
if (size(A) ~= size(B)) 
  error('dim:missmatch', 'A and B should be of same size')
end

V = zeros(N, N_eig);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  LOOP OVER EACH EIGENVALUE TO ESTIMATE THE EIGENVECTOR
%
%  * if lambda is a good guess of a generalized eigenvalue of
%    overdetermined matrix pair (A, B), the the corresponding
%    eigenvector is the right singular vector corresponding to 
%    the smallest singular value. 

for n_eig = 1 : N_eig
  lambda = E_0(n_eig);
  [u, s, v] = svd(A - lambda*B);
  V(:, n_eig) = v(:, N);
end

