

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ev0.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [lam,dlam,beta,X]=ev0(A,B,acc,itmax);
% solves the over- or underdetermined eigenvalue problem
% A*x=lam*B*x
% The computed eigenvalues are guaranteed to be stable under
% perturbations; however, they may be far from the eigenvalues 
% of the exact problem if the latter are highly sensitive.
% However, insensitive eigenvalues are well reproduced.
%
% A,B     matrices of size m times n
% acc     accuracy request - tries to achieve
%         dlam<=acc(1)*|lam|+acc(2)
%         default: acc(1)=acc(2)=10*(m+n)*eps
% itmax   maximal number of iterations tried
%         default: itmax=10
% lam(i)  i-th eigenvalue
% dlam(i) estimated eigenvalue accuracy for exact A,B 
% X(:,i)  i-th eigenvector
% beta    regularization parameter
%         >0: accuracy request probably satisfied 
%         -1: itmax exceeded; eigenvalues might still be useful 
%
function [lam,dlam,beta,X]=ev0(A,B,acc,itmax);

prt=0; % print intermediate results?


% set defaults 
if nargin<3, acc=10*(m+n)*eps*[1,1]; end;
if nargin<4, itmax=10; end;

[m,n]=size(A);
[mb,nb]=size(B);
if mb~=m | nb~=n, error('dimension error'); end;
 
% initialize regularization parameter and scale matrix
beta=eps;
dB=sqrt(diag(B'*B)); % only the diagonal part is needed
dbsum=sum(dB);
if dbsum==0, 
  % case B=0
  lam=inf*ones(n,1);
  dlam=zeros(n,1);
  X=eye(n);
  ier=0;
  return;
end;
ind=(dB==0);dB(ind)=dB(ind)+dbsum;
D=diag(dB);

for it=1:itmax,
  R=qr([B,A;beta*D,zeros(n)]); % contains Q-info in lower triangle
  [mR,nR]=size(R);
  R11=triu(R(1:n,1:n));
  R12=R(1:n,n+1:nR);
  la(1,:)=eig(R12,R11)';
  % simple but costly accuracy check
  % caution: may be pessimistic if ordering of eigenvalues changed!
  % better and less expensive: compute true sensitivities!
  la(2,:)=eig(3*R12,R11)'/3;
  la(3,:)=eig(5*R12,R11)'/5;
  lam=median(la);
  dlam=max(abs(max(la)-lam),abs(lam-min(la)));
  % accuracy check
  fac=max(dlam./(acc(1)*abs(lam)+acc(2)));
  if prt,
    it,beta,fac,la0=real(lam)
    input('next>');
  end;
  if fac<=1, 
    % problem stable
    if nargout>3, [X,D]=eig(R12,R11); end;
    return;
  end;
  % new regularization factor
  % if fac is very large, it may be due to reordering of eigenvalues
  % therefore only fac^(1/3) is applied. 
  % If true sensitivities are used, this could be improved to fac^(1/2)
  beta=max(2*fac,fac^(1/3))*beta;
end;

% maxit exceeded
beta=-1;
  
