function [V, D, D_all] = get_eig_MPA(A, B, E_0, min_iter, max_iter);

%
%  CALLING SEQUENCE:
%
%    [V, D, D_all] = get_eig_MPA(A, B, E_0, min_iter, max_iter);
%
%  SYNOPSIS:
%
%    This function computes generalized eigen values of a pair of 
%    matrices [A, B] using the minimum perturbation approach, see
%    Boutry et al. 2005 (SIAM J. Matrix Anal. App., Vol. 27, No. 2).
%    The maximum number of iterations, and the desired accuracy is
%    provided through the parameters max_iter, and acc respectively.
%    The function returns the generalized eigenvectors as columns of the
%    matrix V, and the generalized eigenvalues are returned as the entries
%    of the diagonal matrix D. The return vector Acc contains the acheived
%    accuray within the max_iter number of iterations. 
%    The following relation is expected to be satisfied in noiseless
%    case:
%                       A*V = B*V*D.
%
%  INPUT:
%
%    A             a over determined rectangular matrix. 
%    B             a over determined rectangular matrix. 
%    E_0           initial approximation of eigenvalues. 
%    min_iter      minumim number of iterations.
%    max_iter      maximum number of iterations performed.
%
%  OUTPUT:
%
%    V              a matrix with the eigenvectors in its columns.
%    D              a diagonal matrix with estimated eigenvalues as entries.
%    D_all          all the eigenvalues in all the iterations. 
%
%  LAST MODIFICATION:
%
%    Jan. 12, 2011, roots of quadratic polynomials are set properly.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK
%

[M, N] = size(A);
if ([M, N] ~= size(B)) 
  error('size_missmatch', 'size of input matrix do not agree');
end

N_eig = length(E_0);
if N_eig > N
  error('size_missmatch', 'more eigen values are requested then possible');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  VARIABLE ARGUMENT INITIALIZATION
%

if nargin < 3
  max_iter = 10;
  acc = 10*eps;
end

if nargin < 4
  acc = 10*eps;
end

D_all = [E_0, nan*ones(N_eig, max_iter)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  INITIALIZE RETURN VARIABLES 
%

D = diag(E_0);
V = zeros(N, N);

if max_iter == 0
  return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ITERATE FOR EACH EIGEN PAIR 

for n = 1  : N_eig 
  lambda = D(n, n);
  for iter = 1 : max_iter

    %% Update the eigenvector
    [U_tmp, D_tmp, V_tmp]  = svd(A - lambda*B);
    v = V_tmp(:, N);

    %% Update the eigenvalue from root of 
    %% a quadratic equation
    Av = A * v; 
    Bv = B * v;
    a = -1 * Av' *Bv;
    b = Av' * Av  -  Bv' * Bv;
    c = Bv' * Av;

    det = sqrt(b^2 - 4*a*c);
    if (sign(b) == sign(real(det)))
      lambda = (-1*b - det)/(2*a);
    else
      lambda = (-1*b - det)/(2*a);
      lambda = c/lambda;
    end
    % lambda = -1*b/(2*a) + sqrt(b^2/(4*a^2) - (c/a));

    %% Break if difference in lambda is not improving 
    if (iter > min_iter)
      lam_diff      = abs(D_all(n, iter) - lambda);
      lam_diff_last = abs(D_all(n, iter) - D_all(n, iter-1));
      if (lam_diff >= lam_diff_last)
        break
      end
    end

    D_all(n, iter+1) = lambda;
  end

  %% Copy results to return vaiables  
  D(n, n) = lambda;
  V(:, n) = v;
end

