function [V, D, D_all] = get_eig_direct(A, B, V_0, D_0, min_iter, max_iter);

%
%  CALLING SEQUENCE:
%
%    [V, D, D_all] = get_eig_direct(A, B, V_0, D_0, min_iter, max_iter);
%
%  SYNOPSIS:
%
%    This function computes generalized eigen values of a pair of 
%    overdetermined matrices [A, B]. It computes the eigenpairs by 
%    alternating a step of inverse iteration and generalized Rayleig quotient 
%    to update the generalized eigenvectors and eigenvalues respectively.
%
%    This is a direct method. In this method, the gradient of a certain
%    objective function is set to zero with respect to the eigenvectors
%    and eigenvalues. Which in turn gives rise to a step of inverse 
%    iteration, and generalized Rayleigh quotient.
%
%    The function returns the generalized eigenvectors as columns of the
%    matrix V, and the generalized eigenvalues are returned as the entries
%    of the diagonal matrix D. The following relation is expected to be 
%    satisfied in noiseless cases: 
%                           A*V = B*V*D.
%
%    For each eigenpair, the method requires O(N^3) operations per iteration. 
%    Convegence is very slow.
%
%    For a pair of square matrices, or if the max_iter is set to zero,
%    the method returns the initial eigenpairs with a warning.
%
%
%  INPUT:
%
%    A             a over determined rectangular matrix. 
%    B             a over determined rectangular matrix. 
%    V_0           the initial eigenvectors in columns.
%    D_0           the initial eigenvalues as entries of a diagonal matrix.
%    min_iter      minumim number of iterations.
%    max_iter      maximum number of iterations.
%
%  OUTPUT:
%
%    V              a matrix with the eigenvectors in its columns.
%    D              a diagonal matrix with estimated eigenvalues as entries.
%    D_all          a matrix of all the eigenvalues over all the iterations. 
%
%  LAST MODIFICATION:
%
%    Jan. 12, 2011, square case return the initial eigenpairs.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK 
%

[M, N] = size(A);

if ([M, N] ~= size(B)) 
  error('dim:missmatch', 'A and B should be of same size')
end

if (M == N || max_iter == 0)
  V     = V_0;
  D     = D_0;
  D_all = diag(D_0);
  warning('if square matrices are given, the initial eigenpairs are returned.')
  warning('if number of iteration is 0, the initial eigenpairs are returned')
  return 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  COMPUTE ALL THE COMPONENTS FROM QR DECOMPOSITION 
%

[Q_tmp, R] = qr([B, A]);
R_11 = triu(R(1:N, 1:N));
R_12 = R(1:N, N+1:2*N);
R_22 = R(N+1:M, N+1:2*N);
[R_0, R, Q, Z] = qz(R_12, R_11);
C = (R_22*Z)'*(R_22*Z);
V_0 = Z'*V_0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  INITITATE RETURN VARIABLES
%

N_eig = size(D_0, 1);
D = zeros(N_eig, N_eig); 
V = zeros(N, N_eig);
D_all = [diag(D_0), nan*ones(N_eig, max_iter)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  REFINE REFINE EACH EIGENVALUE, EIGENVECTOR PAIR 

for n = 1: N_eig

  %% initial eigenpair
  lambda = D_0(n, n);
  x = V_0(:, n);

  R_lambda = R_0 - lambda * R;
  kappa = (norm(R_lambda * x))^2 + real(x'*C*x); 

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  %
  %  ITERATE 
  %

  for iter = 1 : max_iter

    %% Update eigenvector: solve one step of inverse iteration
    x =  ((R_lambda'*R_lambda) + C - kappa*eye(N)) \  x;  
    x = x / norm(x);

    %% Update eigenvalue: through modified Rayleigh Quotient
    lambda = lambda + ((R*x)'*(R_lambda*x)) / ((norm(R*x))^2);
    R_lambda = R_0 - lambda * R;

    %% Update kappa
    kappa = ((norm(R_lambda * x))^2 + real(x'*C*x)); 

    %% Break if difference in lambda is not improving 
    if (iter > min_iter)
      lam_diff      = abs(D_all(n, iter) - lambda);
      lam_diff_last = abs(D_all(n, iter) - D_all(n, iter-1));
      if (lam_diff >= lam_diff_last)
        break
      end
    end

    D_all(n, iter+1) = lambda;
  end

  %% SET REFINED VALUED TO THE RETURN VARIABLES 
  D(n, n) = lambda;
  V(:, n) = Z*x;
end

