function [V, D, D_all] = get_eig_joint(A, B, V_0, D_0, min_iter, max_iter);

%
%  CALLING SEQUENCE:
%
%    [V, D, D_all] = get_eig_joint(A, B, X_0, E_0, max_iter, min_iter);
%
%  SYNOPSIS:
%
%    This function computes generalized eigenpairs of a pair of 
%    overdetermined matrices [A, B]. This routine is a modification of the 
%    least distance approach, see function 'get_eig_least_distance'.
%    In this routine the eigenvalues and the eigenvectors are optimized
%    jointly. This approach results in a much fast convergence.
%    With this approach a parametrized form of the estimated eigenvector
%    is first computed, and then the parameters are estimated to compute
%    the eigenvectors. 
% 
%    The function returns the generalized eigenvectors as columns of the 
%    matrix V, and the generalized eigenvalues are returned as the entries 
%    of the diagonal matrix D. The following relation is expected to be 
%    satisfied in noiseless cases: 
%                          A*V = B*V*D.
%
%    For each eigenpair, the method requires O(N^2) operations per iteration. 
%    The convegence is very fast. 
%
%    For a pair of square matrices, or if the max_iter is set to zero,
%    the method returns the initial eigenpairs with a warning.
%
%  INPUT:
%
%    A             a over determined rectangular matrix. 
%    B             a over determined rectangular matrix. 
%    V_0           the initial eigenvectors in columns.
%    D_0           the initial eigenvalues as entries of a diagonal matrix.
%    min_iter      minumim number of iterations.
%    max_iter      maximum number of iterations.
%
%  OUTPUT:
%
%    V              a matrix with the eigenvectors in its columns.
%    D              a diagonal matrix with estimated eigenvalues as entries.
%    D_all          a matrix of all the eigenvalues over all the iterations. 
%
%  LAST MODIFICATION:
%
%    Jan. 12, 2011, proper documentation and comments.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK
%

[M, N] = size(A);
if ([M, N] ~= size(B)) 
  error('dim:missmatch', 'A and B should be of same size')
end

if (M == N || max_iter == 0)
  V     = V_0;
  D     = D_0;
  D_all = diag(D_0);
  warning('if square matrices are given, the initial eigenpairs are returned.')
  warning('if number of iteration is 0, the initial eigenpairs are returned')
  return 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  COMPUTE ALL THE COMPONENTS FROM QR DECOMPOSITION 
%

[Q_tmp, R] = qr([B, A]);
R_11 = triu(R(1:N, 1:N));
R_12 = R(1:N, N+1:2*N);
R_22 = R(N+1:M, N+1:2*N);
[R_0, R, Q, Z] = qz(R_12, R_11);
E = (R_22*Z);
C = (R_22*Z)'*(R_22*Z);
V_0 = Z'*V_0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  INITIALIZE THE RETURN VARIABLES 
%

N_eig = size(D_0, 1);
D = zeros(N_eig, N_eig); 
V = zeros(N, N_eig);
D_all = [diag(D_0), nan*ones(N_eig, max_iter)];

lam_diff = 0;
lam_diff_last = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SET PARAMETERS FOR OPTIMIZATION
%
%  ** the problem is reduced to optimization  
%     of 3 parameters. Which can be solved in a
%     constant time. We use MATLAB's fminsearch
%     function for this purpose.

param_0 = [1; 0; 0];
opt_options = optimset('fminsearch');
opt_options = optimset(opt_options, 'MaxFunEvals', 1e+32);
opt_options = optimset(opt_options, 'MaxIter', 1e+32);
opt_options = optimset(opt_options, 'GradObj', 'on');
% opt_options = optimset(opt_options, 'FinDiffType', 'central');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  REFINE REFINE EACH EIGENVALUE, EIGENVECTOR PAIR 

for n = 1: N_eig

  %% initial eigenpair
  lambda = D_0(n, n);
  x = V_0(:, n);
  x = x / norm(x);
  method = 0;
  fval_ch_all = [];
  fval_last = 0;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  %
  %  ITERATE 
  %

  for iter = 1 : max_iter

    %%  Compute R_lambda 

    R_lambda = R_0 - lambda* R;
 
    %% Solve the least distance problem for a parametric
    %% representation of the the eigenvector. In particular, 
    %% solve the following linear system for X_param: 
    %%
    %%    | R_lam'*R_lam    x|              |-C*x    R_lam'*R*x   R'*R_lam*v|
    %%    |       x'        0|  X_param  =  | x'*x        0           0     |
    %%
    %%  The above system can be solved fast by decomposing the
    %%  the coefficient matrix as product of upper and lower triangular
    %%  matrices. However R_lambda can be ill-conditioned. Hence the
    %%  smallest diagonal element of R_lambda need to be modified, which
    %%  in turn is a rank 3 update of the original system. With the 
    %%  modified R_lambda, the system can be solved in a stable way, and
    %%  the rank 3 change can be counteracted using Woodbury matrix
    %%  identity at a low cost.

    x_param = solve_psd_update(R_lambda, x, 0, ...
            ([[-1*C*x; x'*x], [R_lambda'*(R*x); 0], [R'*(R_lambda*x); 0]]));
    x_param = x_param(1: N, :);  
    p = x_param(:, 1) - x;
    q = x_param(:, 2) + x_param(:, 3);
    r = sqrt(-1)*(x_param(:, 2) - x_param(:, 3));
 
    %% estimate coefficient of the parameteric representation 
    %% construct the optimization problem,
    %% TODO : this part of the code is getting illegible, 
    %%        for the optimization part make a function.
    R_x = R*[x, p, q, r];
    R_lambda_x = R_lambda*[x, p, q, r];
    E_x  = E*[x, p, q, r];
    x_x = [x, p, q, r]'*[x, p, q, r];

    %% Call one of the search algorithm    
    if (method == 1) 
      warning off
      param = fminunc(@(param) ...
                overdet_obj(param, R_x, R_lambda_x, E_x, x_x), ...
                param_0, ...
                opt_options);
      warning on 
    else 
      param = fminsearch(@(param) ...
                overdet_obj(param, R_x, R_lambda_x, E_x, x_x), ...
                param_0, ...
                opt_options);
    end

    warning off
    [param_dum, fval, exitflag, output, grad] = fminunc(@(param) ...
              overdet_obj(param, R_x, R_lambda_x, E_x, x_x), ...
              param, ...
              opt_options);
    warning on 
    if (iter == 1)
      fval_first = fval;
    end
    fval_change = fval_last - fval;
    fval_last = fval;
    fval_ch_all = [fval_ch_all; fval_change];
    if (abs(fval_change) < 1e-8*fval_first)
      method = 1;
    end
    
    %% update eigenvector
    x = x + ([p, q, r]*param);

    %% Update eigenvalue using modified Rayleigh Quotient
    lambda = lambda + ((R*x)'*(R_lambda*x)) / ((norm(R*x))^2);

    if (iter > 1)
      lam_diff      = abs(D_all(n, iter) - lambda);
      lam_diff_last = abs(D_all(n, iter) - D_all(n, iter-1));
      if (iter > min_iter) & (lam_diff >= lam_diff_last) 
        if method == 0
          method = 0;
        else
          break
        end
      end
    end

    D_all(n, iter+1) = lambda;
  end

  % semilogy(abs(fval_ch_all))
  % hold on 
  %% SET REFINED VALUED TO THE RETURN VARIABLES 
  D(n, n) = lambda;
  V(:, n) = Z*x;
end

%% CHANGE LOG
% 
% * Feb. 08, 2011 -- cases of ill condition R_lambda is covered. 
