
function [V, D, D_all] = get_eig_least_distance(A, B, V_0, D_0, ...
                          min_iter, max_iter);
%
%  CALLING SEQUENCE:
%
%    [V, D, D_all] = get_eig_least_distance(A, B, V_0, D_0, min_iter, max_iter);
%
%  SYNOPSIS:
%
%    This function computes generalized eigenvalues of a pair of overdetermined 
%    matrices [A, B]. This routine is a modification of the direct approach,
%    see function 'get_eig_direct'. A least distance approach is used to
%    replace the step of inverse iteration. There by the complexity of each
%    per iteration, per eigenpair is reduced to O(N^2) from O(N^3), where
%    N is the number of columns of the overdeetemined matrix pairs.
%    Other then computational complexity, there is no difference in the
%    rerults compared to the results of the routine 'get_eig_direct'
%
%    The function returns the generalized eigenvectors as columns of the 
%    matrix V, and the generalized eigenvalues are returned as the entries 
%    of the diagonal matrix D. The following relation is expected to be 
%    satisfied in the noiseless cases: 
%                       A*V = B*V*D.
%
%    For each eigenpair, the method requires O(N^2) operations per iteration. 
%    Convegence is very slow.
%
%    For a pair of square matrices, or if the max_iter is set to zero,
%    the method returns the initial eigenpairs with a warning.
%
%
%  INPUT:
%
%    A             a over determined rectangular matrix. 
%    B             a over determined rectangular matrix. 
%    V_0           the initial eigenvectors in columns.
%    D_0           the initial eigenvalues as entries of a diagonal matrix.
%    min_iter      minumim number of iterations.
%    max_iter      maximum number of iterations.
%
%  OUTPUT:
%
%    V              a matrix with the eigenvectors in its columns.
%    D              a diagonal matrix with estimated eigenvalues as entries.
%    D_all          a matrix of all the eigenvalues over all the iterations. 
%
%  LAST MODIFICATION:
%
%    Jan. 12, 2010, proper documentation.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK
%

[M, N] = size(A);
if ([M, N] ~= size(B)) 
  error('dim:missmatch', 'A and B should be of same size')
end

if (M == N || max_iter == 0)
  V     = V_0;
  D     = D_0;
  D_all = diag(D_0);
  warning('if square matrices are given, the initial eigenpairs are returned.')
  warning('if number of iteration is 0, the initial eigenpairs are returned')
  return 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  COMPUTE ALL THE COMPONENTS FROM QR DECOMPOSITION 
%

[Q_tmp, R] = qr([B, A]);
R_11 = triu(R(1:N, 1:N));
R_12 = R(1:N, N+1:2*N);
R_22 = R(N+1:M, N+1:2*N);
[R_0, R, Q, Z] = qz(R_12, R_11);
C = (R_22*Z)'*(R_22*Z);
V_0 = Z'*V_0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  INITIALIZE RETURN VARIABLES 
%

N_eig = size(D_0, 1);
D = zeros(N_eig, N_eig); 
V = zeros(N, N_eig);
D_all = [diag(D_0), nan*ones(N_eig, max_iter)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  REFINE REFINE EACH EIGENVALUE, EIGENVECTOR PAIR 

for n = 1: N_eig

  %% initial eigenpair
  lambda = D_0(n, n);
  x = V_0(:, n);
  x = x / norm(x);


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  %
  %  ITERATE 
  %

  for iter = 1 : max_iter

    %%  Compute R_lambda 

    R_lambda = R_0 - lambda * R;
    
    %% Solve the least distance problem for the eigenvector,
    %% and normalize. In particular, solve the following linear 
    %% system for X: 
    %%
    %%       | R_lambda'*R_lambda    x|       |-C*x |
    %%       |         x'            0|  X =  | x'*x|
    %%
    %%  The above system can be solved fast by decomposing the
    %%  the coefficient matrix as product of upper and lower triangular
    %%  matrices. However R_lambda can be ill-conditioned. Hence the
    %%  smallest diagonal element of R_lambda need to be modified, which
    %%  in turn is a rank 3 update of the original system. With the 
    %%  modified R_lambda, the system can be solved in a stable way, and
    %%  the rank 3 change can be counteracted using Woodbury matrix
    %%  identity at a low cost.

    x = solve_psd_update(R_lambda, x, 0, ([-1*C*x; x'*x]));
    x = x(1 : N);
    x = x / norm(x);

    %% Update eigenvalue: using modified Rayleigh Quotient

    lambda = lambda + ((R*x)'*(R_lambda*x)) / ((norm(R*x))^2);

    %% Break if difference in lambda is not improving 
    if (iter > min_iter)
      lam_diff      = abs(D_all(n, iter) - lambda);
      lam_diff_last = abs(D_all(n, iter) - D_all(n, iter-1));
      if (lam_diff >= lam_diff_last)
        break
      end
    end

    D_all(n, iter+1) = lambda;
  end

  %% SET REFINED VALUED TO THE RETURN VARIABLES 
  D(n, n) = lambda;
  V(:, n) = Z*x;
end

%% CHANGE LOG
% 
% * Feb. 16, 2011 -- function solve_psd_update is used. 
% * Feb. 08, 2011 -- cases of ill condition R_lambda is covered. 
