function [A, B, X] = get_matrices_from_eigvals(M, N, E);

%
%  CALLING SEQUENCE:
%
%    [A, B, X] = get_matrices_from_eigvals(M, N, E)
%
%  SYNOPSIS:
%
%    This function simulates a pair of matrices [A, B], each with 
%    dimension MxN, with entries of E as their generalized eigen values. 
%    The matrix X contains the eigenvectors in its rows.
%
%    A, B, X, E satisfy the following:
%              A*X = B*X*diag(E)
%
%  NOTE: This function is deprecated. All the functionalities of this
%        function can be found in the function 
%        'get_matrices_from_gen_eigvals'. 
%
%        In the whole software, there is no call to this function. But the
%        function is still kept, it demonstrate some methods on how to
%        simulate a matrix pairs given generalized eigenvalues.
%
%  INPUT:
%
%    M      the number of rows of the out put matrices A, and B.
%    N      the number of columns of the out put matrices A, and B.
%    E      a vector containing the generalized eigen values. 
%
%  OUTPUT:
%
%    [A, B] the matrices which have E as their generalized eigenvalues.
%    X      the matrix with generalized eigenvactors in its rows. 
%
%  LAST MODIFICATION:
%
%    Jan. 13, 2011, the routine is deprecated.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK
%

if M < N
  warning('The routine produces only overdetermined matrices.');
  warning('The routine produces only overdetermined matrices.');
  warning('The number of columns are set equal the number of rows.');
  M = N;
end

[E_m, E_n] = size(E);

if E_n ~= 1
  warning('Considering only the first column of E for the eigen values.');
  E = E(:, 1);
end

if E_m > N
  warning('More eigen values are provided then necessary.');
  warning('Last few in the list are chopped out.');
  E(N+1: E_m) = []; 
end

if E_m < N
  warning('Less eigenvalues then number of columns.');
  warning('Random eigenvalues are appended with given eigen values');
  E = [E ; rand(N - E_m, 1) + sqrt(-1)*rand(N - E_m, 1)];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Set constants
%

ep = 10e-3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Make matrix A as a random square matrix first
%

A = randn(N, N) + sqrt(-1)*randn(N, N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Generate an invertible matix, X
%  [First generate a random one, and then 
%   make sure it is invertble, and columns normalized to 1]
%

X = randn(N, N) + sqrt(-1)*randn(N, N);
[U, D, V] = svd(X);
D = D + ep * eye(N);
X = U * D * V';
X = X *  diag(1 ./ sqrt(sum((abs(X)).^2)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Generate the second matrix B
%

B = A * X * diag(1 ./ E) * pinv(X);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Done for square case
%

if M == N
  return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  For rectangular case, multiply with
%  a full rank rectangular matrix of size MxN
%

Q = rand(M, N) + sqrt(-1) * rand(M, N);
[U, D, V] = svd(Q);
Q = U * (ep + D) * V';
A = Q*A;
B = Q*B;
