function [A, B, X] = get_matrices_from_gen_eigvals(M, N, E);

%
%  CALLING SEQUENCE:
%
%    [A, B, X] = get_matrices_from_gen_eigvals(M, N, E)
%
%  SYNOPSIS:
%
%    This function simulates a pair of matrices [A, B], each with 
%    dimension M x N, with entries of E as their generalized eigen values. 
%    The matrix X contains the eigenvectors in its columns.
%
%    Note that E might contain less that N eigenvalues. Thus the 
%    rsulting return matrices, A and B, are of dimension M x N, and
%    X is of size NXlength(E). 
%
%    The routine works as follows, first two square (N x N) upper triangular 
%    matrices are generated, with diagonals set to produce the given
%    generalized eigenvalue and other entries are random. Further this
%    pair of matrices are pre- and post-multiplied by two unitary matrices
%    in order to produce full square matrices with given eigenvalues.
%    This step is redundant, but kept for better understanding of how 
%    algorithm works from QZ decomposition point of view. Further the
%    pair of matrices are multiplied with a full rank matrix of size
%    M x N to produce overdetermined matrices with specified eigenvalues.
%
%  INPUT:
%
%    M      the number of rows of the out put matrices A, and B.
%    N      the number of columns of the out put matrices A, and B.
%    E      a vector containing the generalized eigen values, 
%           length(E) <= N. 
%
%  OUTPUT:
%
%    [A, B] the matrices which have E as their generalized eigenvalues.
%    X      the matrix with generalized eigenvactors in its rows. 
%
%  LAST MODIFICATION:
%
%    Jan. 12, 2011, changes in comments, no change in functionality.
%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK
%

if M < N
  warning('The routine produces only overdetermined matrices.');
  warning('The routine produces only overdetermined matrices.');
  warning('The number of columns are set equal the number of rows.');
  M = N;
end

[M_e, N_e] = size(E);

if N_e ~= 1
  warning('Considering only the first column of E for the eigenvalues.');
  E = E(:, 1);
end

if M_e > N
  warning('More eigen values are provided then necessary.');
  warning('Last few in the list are chopped out.');
  E(N+1: E_m) = []; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  UPPER TRIANGULAR FORM
%
%  * generate two upper triangular matrices, A, B.
%  * Set the diagonals, in order to get the generalized eigen values.
%

A = triu(randn(N, N) + sqrt(-1)*randn(N, N), 1);
B = triu(randn(N, N) + sqrt(-1)*randn(N, N), 1);

D_a = [E; ones(N - M_e, 1)];
D_b = [ones(M_e, 1); zeros(N - M_e, 1)];

A = A + diag(D_a);
B = B + diag(D_b);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  CREATE FULL MATRIX WITH SAME GEN. EIGEN VALUES 
%
%  * this step is in line with the QZ decomposition of 
%    matrix pairs. Minor modifications can be done to
%    to make this step redundant. Anyway this step is kept
%    in order to demonstrate how the algorithm works 
%    from the QZ decomposition point of view.

Q = randn(N, N) + sqrt(-1)*randn(N, N);
[U_q, D_q, V_q] = svd(Q);

A  = U_q * A * V_q;
B  = U_q * B * V_q;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  RETURN GENERALIZED EIGENVECTORS FROM HERE 
%
%  * using MATLAB's 'eig' command to get the eigenvectors. 
%    Only the first length(E) columns are used, because 
%    other eigenvectors corresponds to 'eigenvalues at infinity'.

[E, X] = eig(A, B);
X = X(:, 1:N_e);

%%%%%%%%%% GENERATE THE RECTANGULAR PART  %%%%%%%%%% 
%
%  * multiply A, and B with a full rank rectangular matrix Q,
%    in order to generate overdetermined matrices. 
%
%  * To assert the full rank of random matrix Q, the singular
%    values of Q are made bigger than zero by at least 'ep'.

ep = 10e-3;

Q = rand(M, N) + sqrt(-1) * rand(M, N);
[U_q, D_q, V_q] = svd(Q);
Q = U_q * (ep + D_q) * V_q';

A = Q*A;
B = Q*B;

