function srf = plot_rect_eig_surf(A, B, xmin, xmax, ymin, ymax, xdelta, ydelta)

% CALLING SEQUENCE:
%
%   srf = plot_rect_eig_surf(A, B, xmin, xmax, ymin, ymax, xdelta, ydelta)
%
% SYNOPSIS:
%
%   This function generates a contour plot of the surface over the Argand that 
%   is minimized in order to compute the generalized eigenpairs of rectangular 
%   matirx pairs. Here the rectangular (overdetermined) matrix pencil is 
%   formed with matrices A, B. The set of parameters xmin, xmax, ymin, ymax, 
%   xdelta, ydelta, are used to evaluate the discrete points in the Argand 
%   plane. The surface generated is:
%
%                || (A - lam*B)*v || 
%     f(lam) =  ---------------------- ,
%                     ||v||
%
%   which in turn is equivalent to computing the smallest eigenvalue of
%   the matrix (A - lam*B), i.e. f(lam) = min(svd(A - lam*B)).
%
%  INPUT:
%
%    A       a rectangular (overdetermined matrix). 
%    B       a rectangular (overdetermined matrix). 
%    xmin    smallet real part on the Argand plane.
%    xmax    largest real part on the Argand plane.
%    ymin    smallet imaginary part on the Argand plane.
%    ymax    largest imaginary part on the Argand plane.
%    xdelta  gap in the real direction of Argand plane.
%    ydelta  gap in the imaginary direction of Argand plane.
%
%  OUTPUT:
%
%    srf     the surface values estimated at the generated grids.
%
%  NOTE:
%
%    The number of contour lines are hardcoded. If contour plot with
%    different numbers of countour lines are required, then produce a 
%    contour plot with the return matrix 'srf' and desired number of 
%    contour lines. 
%
%  AUTHOR / BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SANITY CHECK
%
%  check if dimension of A and B are same. 
%  check if [x, y][min, max, delta] are all consistent 
%            (e.g.  (xmax - xmin) < 0).
%

if ((xmax - xmin) < 0 || (ymax - ymin) < 0)
  error('inconsistent_parameters', 'cannot generated requested grid points');
end

[M, N] = size(A);

if (size(B, 1) ~= M || size(B, 2) ~= N)
  error('size_missmatch', 'size of A and B is not consistent');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  GENERATE GRID POINTS, AND INITIALIZE RETURN MATRIX
%

x_list  = [xmin: xdelta: xmax]; 
y_list  = [ymin: ydelta: ymax]; 
x_N     = length(x_list);
y_N     = length(y_list);

srf = zeros(y_N, x_N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  RUN OVER EACH POINT TO GENERATE THE VALUE
%
%  compute each member of the matrix pencil {A - lambda*B}
%  over the grid of lambda. Save the smallest singular value in the 
%  matrix srf (return value).

for x_n = 1 : x_N
  for y_n = 1 : y_N
    lambda = x_list(x_n) + sqrt(-1)*y_list(y_n);
    S = svd(A - lambda*B);
    srf(y_n, x_n) = S(N); 
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  PLOT WITH CONTOUR COMMAND
%
%  NOTE: the number of contour lines is hardcoded to 100
%  which is also the default value in MATLAB. 

N_cont = 300;
v = linspace(min(srf(:)), max(srf(:)), N_cont);
contour(x_list, y_list, srf, v)
% surf(x_list, y_list, srf)

