function [V, D] = oeig(A, B, param)

%
%  OEIG       eigenvalue and eigenvector of rectangular matrix pencil.
%
%  NOTE: The number of eigenvalues can be less than minimum of the row and
%        column size. 
%   
%  E = OEIG(A,B) is a vector containing the generalized eigenvalues
%     of overdetermined matrices A and B.
% 
%  [V,D] = OEIG(A,B) produces a diagonal matrix D of generalized
%     eigenvalues and a full matrix V whose columns are the
%     corresponding eigenvectors of the matrix pair A, B.
%  
%  [V,D] = OEIG(A,B,p) uses the parameters specified in p (as spefied in 
%     oeig_param.m) to produce a diagonal matrix D of generalized eigenvalues 
%     and a full matrix V whose columns are the corresponding eigenvectors of 
%     the matrix pair A, B.
%
%  p = OEIG(0) returns the default parameter structure, to be used as the third
%     argument. 
%
%  OEIG() runs a demo program by simulating some random overdetermined 
%     matrices with known eigenvalues.
%


%
%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SET PATH
%

path(path, genpath(pwd));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  If there is no argument given, then
%     1. Display help
%     2. Run a demo program
%     3. Return
%

if nargin == 0
  help oeig;
  fprintf('Running a demo program ... \n')
  oeig_demo;
  V = 'The results are shown in the Figure';
  return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  SET DEFAULT PARAMETERS
%

param_def.iter            = [20, 30];
param_def.huge            = 1e06;
param_def.ini_method      = 'ini_method 2';
param_def.esti_method     = 'joint';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  If there is only one input argument,
%  return the default  parameters.
%

if nargin == 1
  V = param_def;
  return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  If there are user specifed parameter, 
%  then overwrite the default
%

if nargin == 3
  f_names = fieldnames(param);
  for i = 1 : length(f_names);
    f_name = f_names(i);
    param_def.(f_name{:}) = getfield(param, f_name{:});
  end
end
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  PARAMETER AND METHOD SETTING DONE.
%  EXECUTING THE ROUTINES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  GET STARTING EIGENPAIRS 
%
%  * depending on the method selected for estimating 
%    the starting eigenpairs, the relevent routine is
%    called.
%
%  * if one have a good guess of starting eigenpairs, 
%    then set the eigenvectors in the columns of X_0,
%    and eigenvalues in the diagonal matrix D_0. And
%    skip the in-bulit routines. Do not forget to set
%    ini_method to 'ini_method 0'.
%
%  * if one have a good guess of starting eigenvalues 
%    only, and want only the corresponding eigenvectors
%    to be computed, the set ini_method to 'ini_method 4',
%    and set the eigenvalues in the column matrix E_0.

switch param_def.ini_method
  case {'ini_method 0'}
    if (~exist('X_0', 'var') || ~exist('D_0', 'var'))
      error('missing:data', 'initial eigen pairs are missing')
    end
  case {'ini_method 1'}
    [X_0, D_0] = initial_eigenpairs_using_random_rows(A, B);
  case {'ini_method 2'}
    [X_0, D_0] = initial_eigenpairs_using_QR_decomp(A, B);
  case {'ini_method 3'}
    [X_0, D_0] = initial_eigenpairs_using_square_form(A, B);
  case {'ini_method 4'}
    if (~exist('E_0', 'var'))
      error('missing:data', 'initial eigen pairs are missing')
    end
    X_0 = initial_eigenvectors_from_smallest_singularvalue(A, B, E_0);
    D_0 = diag(E_0);
  otherwise
    error('invalid:option', 'the option for initial eigenpairs is not valid')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  FILTER EIGENPAIRS BEFORE ITERATIVE ALGORITHMS
%
%  thr   drop an eigenpair if the abs(1/eigenvalue) is smaller than thr.
%  N_e   number of initial eigenpairs to process.

[X_0, D_0] = filter_initial_eigenpairs(X_0, D_0, 1/param_def.huge);
N_e  = length(diag(D_0));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ESTIMATE EIGENPAIRS : MINIMAL PERTURNATION APPROACH, MPA
%  ESTIMATE EIGENPAIRS : REGULARIZATION APPROACH
%  ESTIMATE EIGENPAIRS : LEAST DISTANCE FOR LOW COMPLEXITY
%  ESTIMATE EIGENPAIRS : JOINT OPTIMIZATION OF EIGENVALUES AND VECTORS
%

min_iter = param_def.iter(1); 
max_iter = param_def.iter(2); 

switch param_def.esti_method
  case {'MPA'}
    [V, D, D_all] = get_eig_MPA(A, B, diag(D_0), max_iter);
  case {'direct'}
    [V, D, D_all] = get_eig_direct(A, B, X_0, D_0, max_iter);
  case {'ldist'}
    [V, D, D_all] = get_eig_least_distance(A, B, X_0, D_0, max_iter);
  case {'joint'}
    [V, D, D_all] = get_eig_joint(A, B, X_0, D_0, max_iter, min_iter);
    %% perform last two-three iterations with least distance approach
    %% in order to get a better precision.
    [V, D, D_all2] = get_eig_least_distance(A, B, V, D, 3, 3);
    D_all = [D_all, D_all2];
  otherwise
    error('invalid:option', ...
          'the option for eigenpair estimation method is not valid')
end

% plot(abs(D_all.'))
% pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  If there is only one output required,
%  return the eigenvalues only.
%

if (nargout == 1 || nargout == 0)
  V = diag(D);
  return 
end



% CHANGE LOG
%
% * Feb. 08, 2011: max_iter, min_iter is changed to two element vector.
% * Jan. 28, 2011: change in the help. 
% * Jan. 27, 2011: min_iter is set 
% * Jan. 27, 2011: default parameters are set
% * Jan. 26, 2011: copied one of the driver to generate this function. 
