function X = solve_psd_update(R, a, p, B)

%  CALLING SEQUENCE:
%
%    X = solve_psd_update(R, a, p, B)
%
%  SYNOPSIS:
%
%    Solves for a linear system AX = B, where the ceofficient matrix A
%    is positive (semi) definite matrix constructed as:
%
%                     | R'R  a  |
%                A  = |  a'  p  |
%
%    where R is an upper triangular matrix, a is a vector, and p is a
%    scalar. Many a time, the matrix R is ill-conditioned, but A is not.
%    However, one would like to exploit the structure of R in order to
%    solve with low complexity. 
%
%    We modify the diagonal element of R to make it well conditioned, and
%    later we update the solution using Woodbury matrix identity to get
%    the solution for the problem.
%
%    The size of return matrix (vector) X is same as the size of B. If B is
%    a matrix, then solution for each column is returned in the corresponding
%    column of X.
%
%    Note that the present implementation is limited to only one 
%    perturnbation in the dignal of R. 

%  BUG REPORT:
%
%    saptarshi.das@univie.ac.at
%    

[M , N] = size(R);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ** As a regularization step, find the smallest diagonal 
%     element of R, and make it big.
%
%  ** The matrix S is rank one update of R.
%

[min_d, k] = min(abs(diag(R)));
roh = max(abs(R(:)));
e_lam = zeros(N, 1); e_lam(k) = 1;
S = R + roh*e_lam*e_lam'; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ** Solve the system with approximate matrix S,
%     i.e.,
%           |S'*S    a|
%           |  a'    p|  X = B
%
%  ** First compute the LU factors of the coefficient matrix, 
%     which can be done in a in a cheap way. Further use the
%     LU factors to compute the solution. Call this intermediate
%     solution X_int.

S_a = S' \ a;
Coeff_L = [[S', zeros(N, 1)]; [S_a', 1*sqrt(S_a'*S_a)]];
Coeff_U = [[S, S_a]; [zeros(1, N), -1*sqrt(S_a'*S_a)]];
A_inv_B = Coeff_U \ (Coeff_L \ B);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ** Compute the rank 3 difference (U*D*V') of the 
%     coefficient matrix, between the original system 
%     in terms of R, and the stable system in terms of S.
%

U = [[S(k, :)', e_lam, e_lam]; [0, 0, 0]];
D = diag([roh, roh, -1*roh^2]);
V = [[e_lam, S(k, :)', e_lam]; [0, 0, 0]];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  ** Solve for the original system (coefficient matrix 
%     in terms of R) by correcting the intermediate 
%     approximate solution (obtained using the coefficient 
%     matrix in terms of S) X_int, using Woodbury matrix 
%     identity. 
%     

A_inv_U = Coeff_U \ (Coeff_L \ U);
X = A_inv_B - (A_inv_U * pinv(-1*inv(D) + V'*A_inv_U ) * V'*A_inv_B);


%  CHANGE LOG
%
% * Feb, 15 2011 -- First lines of codes. 
